@@ -1,8 +1,59 @@
+# typing
+from collections.abc import Iterable
+
+# gamut
 from .utils import catch_keyboard_interrupt
 
 
+def print_success(text) -> None:
+    """ Prints green-colored message """
+    print(f"\N{check mark} \033[32;1m{text}\033[0m")
+
+
+def print_warning(text) -> None:
+    """ Prints yellow-colored message """
+    print(f"\N{warning sign} \033[33;1m{text}\033[0m")
+
+
+def print_error(error) -> None:
+    """ Prints red-colored message """
+    print(f"\N{skull} \033[31;1m{error}\033[0m")
+    exit()
+
+
+def is_kivy_installed() -> bool:
+    """ Check if kivy is installed """
+    import pkg_resources
+    return 'kivy' in list(map(lambda x: x.key, pkg_resources.working_set))
+
+
+def gui() -> None:
+    """ Entry points to launch GUI """
+    # Check if kivy has been installed
+    if not is_kivy_installed():
+        import subprocess
+        import threading
+        from sys import executable
+
+        # try installing on separate thread and wait until done
+        print_warning("Attempting to install missing dependencies...")
+        t = threading.Thread(target=lambda: subprocess.run([executable, '-m', 'pip', 'install', 'kivy[base]']))
+        t.start()
+        t.join()
+
+        # run with --retry flag to prevent infinite recursion (see conditional in cli())
+        print_success("Running again...")
+        subprocess.run(['gamut', '--gui', '--retry'])
+        exit()
+
+    print_success('Launching user interface...')
+    from .gui import GUI
+    GUI().run()
+
+
 @catch_keyboard_interrupt()
 def cli():
+    """ CLI entry point """
     from argparse import ArgumentParser
     from os import chdir, makedirs, remove, mkdir, getcwd
     from os.path import realpath, join, exists, splitext, basename
@@ -17,17 +68,7 @@ def cli():
     # HELPER FUNCTIONS
     # ------------------------------------- #
 
-    def print_success(text):
-        print(f"\N{check mark} \033[32;1m{text}\033[0m")
-
-    def print_warning(text):
-        print(f"\N{warning sign} \033[33;1m{text}\033[0m")
-
-    def print_error(error):
-        print(f"\N{skull} \033[31;1m{error}\033[0m")
-        exit()
-
-    def parse_params(raw_params):
+    def parse_params(raw_params: Iterable) -> dict:
         params = {}
         if not raw_params:
             return params
@@ -45,15 +86,15 @@ def cli():
             params[key] = value
         return params
 
-    def safe_chdir(dest):
+    def safe_chdir(dest: str) -> None:
         if not exists(dest):
             return
         chdir(dest)
 
-    def abs_path(path, ext):
+    def abs_path(path: str, ext: str) -> str:
         return realpath(splitext(path)[0] + ext)
 
-    def write_file(path, obj, ext, script_block):
+    def write_file(path: str, obj: object, ext: str, script_block: str) -> None:
         out = abs_path(path, ext)
         if not NO_CACHE:
             CACHED_OBJECTS[out] = obj
@@ -62,13 +103,13 @@ def cli():
                 remove(out)
             obj.write(out)
 
-    def clean_path(file):
+    def clean_path(file: str) -> str:
         path = realpath(file)
         if not exists(path):
             print_error(f"{path} does not exist.")
         return path
 
-    def check_subdirectories(script):
+    def check_subdirectories(script: str) -> None:
         subdirs = [AUDIO_DIR, CORPUS_DIR, MOSAIC_DIR]
         missing = []
         for subdir in subdirs:
@@ -96,7 +137,7 @@ def cli():
         if answer.lower() in ['n', 'no']:
             exit()
 
-    def define_directories(root):
+    def define_directories(root: str) -> None:
         global ROOT_DIR
         global MOSAIC_DIR
         global CORPUS_DIR
@@ -112,7 +153,7 @@ def cli():
         TEST_NAME = 'test'
         TEST_SCRIPT_DIR = join(SCRIPTS_DIR, f'{TEST_NAME}.json')
 
-    def create_new_template(template):
+    def create_new_template(template: str) -> None:
         chdir(ROOT_DIR)
         out_path = splitext(template)[0]
         name = basename(out_path)
@@ -172,6 +213,12 @@ def cli():
     parser.add_argument('--test',
                         action='store_true',
                         help='runs GAMuT package test.')
+    parser.add_argument('--gui',
+                        action='store_true',
+                        help='open GAMuT graphical user interface.')
+    parser.add_argument('--retry',
+                        action='store_true',
+                        help='Flag to signal attempt to re-run UI after kivy installation attempt.')
     parser.add_argument('--no-check',
                         action='store_true',
                         help='Disable directory checking when reading a script')
@@ -195,6 +242,9 @@ def cli():
     parser.add_argument('-p', '--play',
                         action='store_true',
                         help="enable audio playback after script runs")
+    parser.add_argument('--no-verbose',
+                        action='store_true',
+                        help="disables console printing")
     parser.add_argument('--skip',
                         nargs='+',
                         help="skip one or more blocks from the script")
@@ -233,26 +283,38 @@ def cli():
     if len(SKIP_WRITE) > 0 and NO_CACHE:
         print_error("You can't use --skip-write and --no-cache at the same time.")
 
+    if args.no_verbose:
+        from .sys import set_vebosity
+        set_vebosity(False)
+
     # ------------------------------------- #
     # PRINT GAMUT PACKAGE VERSION
     # ------------------------------------- #
     if args.version:
         print_success(f'GAMuT v{pkg_resources.get_distribution("gamut").version}')
 
+    # ------------------------------------- #
+    # OPEN GRAPHICAL USER INTERFACE
+    # ------------------------------------- #
+    elif args.gui:
+        if not is_kivy_installed() and args.retry:
+            print('Unable to install kivy library. Please visit https://kivy.org/doc/stable/gettingstarted/installation.html#install-pip to install it manually and try again afterwards.')
+        import subprocess
+        subprocess.run(['gamut-ui'])
+
     # ------------------------------------- #
     # RUN PACKAGE TEST
     # ------------------------------------- #
     elif args.test:
-        TEST_DIR = '.gamut-test'
+        TEST_DIR = realpath('.gamut-test')
+        if exists(TEST_DIR):
+            rmtree(TEST_DIR)
         mkdir(TEST_DIR)
-        TEST_DIR = realpath(TEST_DIR)
-        try:
-            safe_chdir(TEST_DIR)
-            run(['gamut', '--init'])
-            run(['gamut', '--script', join(basename(SCRIPTS_DIR), f"{TEST_NAME}.json"), '--no-cache'])
-            print_success('GAMuT test successful!')
-        except Exception as e:
-            print(e)
+        safe_chdir(TEST_DIR)
+        print_success("Running test...")
+        run(['gamut', '--init', '--no-verbose'])
+        run(['gamut', '--script', join(basename(SCRIPTS_DIR), f"{TEST_NAME}.json"), '--no-cache', '--no-verbose'])
+        print_success("Done")
         rmtree(TEST_DIR)
 
     # ------------------------------------- #
@@ -315,9 +377,8 @@ def cli():
                             copyfileobj(r.raw, f)
             except:
                 print('\tWarning: Unable to download audio examples')
-
-        print_success(
-            f"Your GAMuT project folder is ready!\nTry running:\n\tgamut --script scripts/{TEST_NAME}.json --play")
+        if not args.no_verbose:
+            print_success(f"Your GAMuT project folder is ready! Try running:\n\tgamut --script scripts/{TEST_NAME}.json --play")
 
     # ------------------------------------- #
     # CREATE TEMPLATE
